/*
 * MultiMAIL offline mail reader
 * 

   Written by Toth Istvan (stoty@vma.bme.hu)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include "bwreply.h"
#include "misc.h"
#include "resource.h"

// current
bwreply::bwreply(mmail *mmA)
{
	mm = mmA;
	init();
};

bwreply::~bwreply()
{
	if(replyExists)
		cleanup();
};

int bwreply::init()
{
	uplHeader = new UPL_HEADER;

	replyText = new char[1];

	replyPacketName = newFileName(mm->resourceObject->get(bwPacketName));
	
	chdir(mm->resourceObject->get(bwReplyDir));

	if(access(replyPacketName, W_OK)==0)
		replyExists =1;
	else
		replyExists =0;
	
	//if exists, read the letters
	
	if(replyExists)
	{
		uncompress();
		readUpl();
		currentLetter = 1;
	}
	else
	{
		uplListHead = NULL;
		noOfLetters = 0;
		currentLetter =0;	
	};	 	 
				
	return 0;	

};

int bwreply::cleanup()
{
	upl_list *curr;
	upl_list *next;
	delete uplHeader;
	delete replyText;
	delete replyPacketName;

	curr = uplListHead;
	
	for(int c=1; c<=noOfLetters; c++)
	{
		next = curr->nextRecord;
		delete curr;
		curr = next;
	};
	return 0;
};

char *bwreply::newFileName(const char *bwFile)
{
	int c;
	char *tmp;
	char suffix[5];
	
	if(tolower(bwFile[0]) == bwFile[0]) // lower-case
		suffix = ".new";
	else
		suffix = ".NEW";
	
	for(c=0; bwFile[c]!='.'; c++)
	tmp = new char[c+1+5];
	for(int d=0; d < c; tmp[d]=bwFile[d++]);
	strcpy(tmp+c, suffix);
	
	return tmp;
};

int bwreply::uncompress()
{
	char	command[255];
	 
	if(chdir(mm->resourceObject->get(bwUpWorkDir)))
		fatalError("\nin bwreply::uncompress");
	system("rm *");
	
	strcpy(command, mm->resourceObject->get(bwUncompressCommand));
	strcat(command, " ");
	strcat(command, mm->resourceObject->get(bwReplyDir));
	strcat(command, "/");
	strcat(command, replyPacketName);
	
	system(command);
	return 0;
};

int bwreply::readUpl()
{

	char uplName[14];
	char *tmp;
	int uplLength;
	int *broaf;

	upl_list *prevUplList;
	upl_list *currUplList;

	chdir(mm->resourceObject->get(bwUpWorkDir));
	tmp=findBaseName(replyPacketName);
	strcpy(uplName,tmp);
	strcat(uplName,".UPL");
	delete tmp;
	
	if(!(uplFile=fopen(uplName, "rw")))
		fatalError("error opening UPL");
	
	//rewind(uplFile);  //just because you're paranoid...
	if(!fread(uplHeader, sizeof(UPL_HEADER), 1, uplFile))
		fatalError("error reading UPL file");
	
	fseek(uplFile, 0, SEEK_END);
	uplLength = ftell(uplFile);
	
	noOfLetters = (int) 
	((uplLength-uplHeader->upl_header_len) / uplHeader->upl_rec_len);
	
	fseek(uplFile, uplHeader->upl_header_len, SEEK_SET);
	uplListHead = new upl_list;
	fread(&(uplListHead->uplRec), sizeof(UPL_REC), 1, uplFile);
	broaf = (int*) uplListHead->uplRec.user_area; 
	*broaf= (int) getFileLen((char*)uplListHead->uplRec.filename)+1;
	
	uplListHead->nextRecord=NULL;
	prevUplList = uplListHead;	

	
	for(int c=1; c<=noOfLetters-1; c++)
	{
		fseek(uplFile, uplHeader->upl_header_len+c*uplHeader->upl_rec_len, 
		SEEK_SET);
		currUplList = new upl_list;
		fread(&(currUplList->uplRec), sizeof(UPL_REC), 1, uplFile);
		broaf=(int*)(currUplList->uplRec.user_area); 
		*broaf = (int) getFileLen((char*)currUplList->uplRec.filename)+1;
	
		prevUplList->nextRecord = currUplList;
		prevUplList = currUplList;
	};
	return 0;
};

	
int bwreply::getNoOfAreas()
{
	return 1;
};

int bwreply::resetAll()
{
	cleanup();
	init();
	return 0;
};

area_header *bwreply::getNextArea()
{
	resetLetters();
	replyAreaHeader = new area_header(mm, 1, "REPLY", "REPLIES", 
		"Letters written by You", bwrep, noOfLetters, 0, 0);
	return replyAreaHeader;
};

int bwreply::selectArea(int ID)
{
	if(ID!=1) 
		return -1;
	resetLetters();
	return 0;
};

int bwreply::resetLetters()
{
	currentLetter = 1;
	uplListCurrent = uplListHead;
	return 0;
};

int bwreply::getNoOfLetters()
{
	return noOfLetters;
};

letter_header *bwreply::getNextLetter()
{
	int areaNo;
	time_t unixTime;
	char *date;
	letter_header *newLetter;
	
	unixTime = time(NULL);
	
	date = ctime(&unixTime);
	
	areaNo=1;
	for(int c=1; c <= mm->areaList->noOfAreas(); c++)
	{
		if(!strcmp(mm->areaList->getName(c), (char*)uplListCurrent->uplRec.echotag))
			areaNo=c;
	};
	
	newLetter = new letter_header(mm, (char*)uplListCurrent->uplRec.subj, 
	(char*)uplListCurrent->uplRec.to, 
	(char*)uplListCurrent->uplRec.from, date , 0, 0, currentLetter, 
	areaNo, 0,0 ,0,0,0,0, *(int*)(uplListCurrent->uplRec.user_area), this);
	
	uplListCurrent = uplListCurrent->nextRecord;
	
	currentLetter++;
	return newLetter;
};

char* bwreply::getBody(int ID)
{
	FILE *replyFile;
	upl_list *actUplList;

	delete replyText;
	
	actUplList = uplListHead;
	for(int c=1; c<=ID-1; c++)
		actUplList = actUplList->nextRecord;
	
	chdir(mm->resourceObject->get(bwUpWorkDir));
	replyFile = fopen((char*)actUplList->uplRec.filename, "r"); // !! check it !

	replyText = new char[*(int*)(actUplList->uplRec.user_area)];
	fread(replyText, *(int*)(actUplList->uplRec.user_area), 1, replyFile);
	replyText[(*(int*)(actUplList->uplRec.user_area))+1]='\0';
	fclose(replyFile);
	return replyText;
};

int bwreply::enterLetter(letter_header* newLetter)
{
	int *broaf;
	upl_list *workList;
	upl_list *newList;

	char *newLetterFileName;
	char copyCommand[255];
	
	// copy the file to a unique name
	if(chdir(mm->resourceObject->get(bwUpWorkDir))!=0)
		fatalError("could not cd to workdir\n\n");
	
	newLetterFileName=freeFileName(); 
	sprintf(copyCommand, "/usr/bin/todos < ../%s > ./%s", 
	mm->resourceObject->get(bwReplyFileName), newLetterFileName);
	system(copyCommand);
	
	workList = uplListHead;
	
	for(int c=1; c<noOfLetters; c++) //go to last elem
		workList = workList->nextRecord; 
	
	newList = new upl_list;
	
	memset((void*) &(newList->uplRec), 0, sizeof(UPL_REC));
	
	
	// fill the fields of UPL_REC
	strncpy((char*)newList->uplRec.from, newLetter->getFrom(), 36);
	strncpy((char*)newList->uplRec.to, newLetter->getTo(), 36);
	strncpy((char*)newList->uplRec.subj, newLetter->getSubject(), 72);
	strncpy((char*)newList->uplRec.echotag, mm->resourceObject->get(bwEchoTag), 21);	
	strncpy((char*)newList->uplRec.filename, newLetterFileName, 13);
	newList->uplRec.unix_date=(long)time(NULL);
	if(netAddr.zone!=0)
	{
		newList->uplRec.msg_attr|=UPL_NETMAIL;
		newList->uplRec.destzone=netAddr.zone;
		newList->uplRec.destnet=netAddr.net;
		newList->uplRec.destnode=netAddr.node;
		newList->uplRec.destpoint=netAddr.point;
		netAddr.zone=0;
	}
	else	
	{	
		newList->uplRec.destzone=0;
		newList->uplRec.destnet=0;
		newList->uplRec.destnode=0;
		newList->uplRec.destpoint=0;
		newList->uplRec.netmail_attr=0;
	};
	
	broaf=(int*)(newList->uplRec.user_area);
	*broaf = (int) getFileLen((char*)newList->uplRec.filename)+1;

	if(workList == NULL)
		uplListHead = newList;
	else
		workList->nextRecord = newList;

	delete newLetterFileName;
	noOfLetters++;
	return 0;
	
};

int bwreply::enterNetDest(net_address netAddress)
{
	netAddr=netAddress;
	return 0;
};     

int bwreply::killLetter(int letterNo)
{
	upl_list *actUplList;
	upl_list *tmpUplList;

	if( (noOfLetters == 0) || (letterNo < 1) || (letterNo > noOfLetters) )
		fatalError(" bwreply::killLetter ::fukka ! roszzat akarsz torolni!: ");

	//cd to workdir
	if(chdir(mm->resourceObject->get(bwUpWorkDir)))
		fatalError(" bwreply::killLetter ::could not cd to workdir");
	
	// deleteing the 1st letter
	if(letterNo==1)
	{
		tmpUplList = uplListHead;
		uplListHead = uplListHead->nextRecord;
	}
	else 
	{
		actUplList = uplListHead;
		for(int c=1; c<=letterNo-2; c++)
			actUplList = actUplList->nextRecord;
	
		//deleting the last letter
		if(letterNo==noOfLetters)
		{
			tmpUplList = actUplList->nextRecord;
			actUplList->nextRecord = NULL;
		}
		else	// deleting from the middle
		{
			tmpUplList =  actUplList->nextRecord;
			actUplList->nextRecord= actUplList->nextRecord->nextRecord;
		};
	}
	noOfLetters--;
	unlink( (char*)tmpUplList->uplRec.filename);
	delete tmpUplList;
	resetLetters();
	return 0;
};	

area_header *bwreply::refreshArea()
{
	replyAreaHeader = new area_header(mm, 1, "REPLY", "REPLIES",
    "Letters written by you", bwrep, noOfLetters, 0, 0);
	resetLetters();
	return replyAreaHeader;
}; 

int	bwreply::makeReply()
{
	upl_list *actUplList;
	UPL_HEADER newUplHeader;
	FILE *uplFile;
	char uplFileName[13];
	char *tmp;
	char tmp1[255],command[255];

	// cd to upload dir
	if(chdir(mm->resourceObject->get(bwUpWorkDir)))
		fatalError("\n Could not cd to bwupworkdir in bwreply::makeReply\n\n");

	// If no messages, delete file
	if(noOfLetters==0)
	{	
		strcpy(tmp1, mm->resourceObject->get(bwReplyDir));
		strcat(tmp1, "/");
		strcat(tmp1, replyPacketName);
		unlink(tmp1);
		return 0;
	}
	
	// fill the UPL_HEADER struct
	memset(&newUplHeader,0,sizeof(UPL_HEADER));
	
	strcpy((char*)newUplHeader.vernum, "0.1 alpha");
	for(int c=0; newUplHeader.vernum[c] ;newUplHeader.vernum[c++]+=10); 
	newUplHeader.reader_major=0;
	newUplHeader.reader_minor=1;
	strncpy((char*)newUplHeader.reader_name, "MultiMail Mail Reader",80);
	newUplHeader.upl_header_len=sizeof(UPL_HEADER);
	newUplHeader.upl_rec_len=sizeof(UPL_REC);
	strncpy((char*)newUplHeader.loginname, mm->resourceObject->get(bwLoginName),44);
	strncpy((char*)newUplHeader.aliasname, mm->resourceObject->get(bwAliasName),44);
	strncpy((char*)newUplHeader.reader_tear, "Multimail",16);

	// find out the name of the upl file
	tmp = findBaseName(mm->resourceObject->get(bwPacketName));
	strcpy(uplFileName, tmp);
	strcat(uplFileName, ".UPL");
	delete tmp;

	// create the new UPL file
	uplFile=fopen(uplFileName,"w");
	//!! no check yet
	fwrite(&newUplHeader ,sizeof(UPL_HEADER), 1, uplFile);

	actUplList = uplListHead;

	for(int c=0; c<noOfLetters; c++)
	{
		fwrite(&(actUplList->uplRec), sizeof(UPL_REC), 1 , uplFile);
		actUplList=actUplList->nextRecord;
	};

	fclose(uplFile);
	
	//pack the files
	
	strcpy(command, mm->resourceObject->get(bwCompressCommand));
	strcat(command, " ");
	strcat(command, mm->resourceObject->get(bwReplyDir));
	strcat(command, "/");
	strcat(command, replyPacketName);
	strcat(command, " *");
	system(command);  //!! check 4 errors !!
	
	// clean up the work area
	//!! danger !!
	system("rm *");
	return 0;
};

char* bwreply::freeFileName()
{
	char testFileName[255];
	for(int c=0; c<=999; c++)
	{
		sprintf(testFileName,"BW.%03d",c);
		if(access(testFileName, R_OK)==-1)
			return strdupplus(testFileName);
	}
	fatalError("No more than 999 replies allowed in a packet !");
};
