/*
 * MultiMAIL offline mail reader
 * 

   Written by Toth Istvan (stoty@vma.bme.hu)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <stdio.h>
#include <strings.h>
#include <iostream.h>
#include <time.h>
#include "bw.h"
#include "bwreply.h"
#include "misc.h"
#include "mmail.h"

// ---------------------------------------------------------------------------
// Letter header methods
// ---------------------------------------------------------------------------

letter_header::letter_header(mmail *mmA, const char * subjectA, const char *toA, 
	const char *fromA, const char* dateA, const int replyToA, 
	const int replyAtA, const int LetterIDA, const int AreaIDA, 
	const int readA, const int recievedA, const int privatA, 
	const int repliedA, const int killedA, const int originalIDA, 
	const int lengthA, specific_driver *driverA)
	{
		mm = mmA;
        subject = strdupplus(subjectA);
		to = strdupplus(toA);
		from = strdupplus(fromA);
		date = strdupplus(dateA);
		replyTo = replyToA;
		replyAt = replyAtA;
		LetterID = LetterIDA;
		AreaID = AreaIDA;
		recieved = recievedA;
		privat = privatA;
		replied = repliedA;
		killed = killedA;
		originalID = originalIDA;	
		length = lengthA;
		driver = driverA;
		read=readA; //suppress warning
		read = mm->driverList->getReadObject(driver)->getRead(AreaID, LetterID);
	};

letter_header::~letter_header()
{
	delete subject;
	delete to;
	delete from;
	delete date;
};
                                    
const char  *letter_header::getSubject()
{
	return subject;
};

const char  *letter_header::getTo()
{
	return to;
};
	
const char  *letter_header::getFrom()
{
	return from;
};

const char	*letter_header::getDate()
{
	return date;
};
	
const int   letter_header::getReplyTo()
{
	return replyTo;
}; 

const int   letter_header::getReplyAt()
{
	return replyAt;
};

const int   letter_header::getLetterID()
{
	return LetterID;
};

const int   letter_header::getAreaID()
{
	return AreaID+mm->driverList->getOffset(driver);
};        
	
const int   letter_header::getRead()
{
	return read;
};
	
const int   letter_header::getRecieved()
{
	return recieved;
};
	
const int   letter_header::getPrivate()
{
	return privat;
};
	
const int   letter_header::getReplied()
{
	return replied;
};
	
const int   letter_header::getKilled()
{
	return killed;
};
	
const int   letter_header::getOriginalID()
{
	return originalID;
};

const int   letter_header::getLength()
{
	return length;
};

specific_driver *letter_header::getDriver()
{
	return driver;
};

int letter_header::setRead()
{
	read = 1;
	mm->driverList->getReadObject(driver)->setRead(AreaID, LetterID, read);
	return 0;
};
	
int letter_header::setUnread()
{
	read =0;
	return 0;
};

// ---------------------------------------------------------------------------
// Letterlist methods
// ---------------------------------------------------------------------------

letter_list::letter_list(mmail *mmA, int areaNumberA)
{
	mm = mmA;

	areaNumber = areaNumberA;
	
	init();
}

int letter_list::init()
{	
	driver = mm->driverList->getDriver(areaNumber); 
	
	driver->selectArea(areaNumber-mm->driverList->getOffset(driver));
	
	noOfLetters = driver->getNoOfLetters();
	
	letterHeader = new (letter_header*)[noOfLetters+1];

	driver->resetLetters();

	for(int c=1; c <= noOfLetters; c++)
	{
		letterHeader[c] = driver->getNextLetter();
	};

	currentLetter = 1;
	return 0;	
};

letter_list::~letter_list()
{
	cleanup();
};

int letter_list::cleanup()
{
	for(int c=1; c<=noOfLetters; c++)
		delete letterHeader[c];
	delete letterHeader;
	return 0;
};

int letter_list::noOfLetter()
{
	return noOfLetters;
};

int letter_list::gotoLetter(int newLetter)
{
	if((newLetter > 0) && (newLetter <=noOfLetters))
		currentLetter=newLetter;
	return newLetter;
};

const char*	letter_list::getSubject()
{
	return letterHeader[currentLetter]->getSubject();
};

const char* letter_list::getFrom()
{
	return letterHeader[currentLetter]->getFrom();
};

const char* letter_list::getTo()
{
	return letterHeader[currentLetter]->getTo();
};

const char*	letter_list::getDate()
{
	return letterHeader[currentLetter]->getDate();
};

const int	letter_list::getReplyTo()
{
	return letterHeader[currentLetter]->getReplyTo();
};

const int	letter_list::getReplyAt()
{
	return letterHeader[currentLetter]->getReplyAt();
};

const int	letter_list::getLetterID()
{
	return letterHeader[currentLetter]->getLetterID();
};

const int	letter_list::getAreaID()
{
	return letterHeader[currentLetter]->getAreaID();
};

const int	letter_list::getRead()
{
	return letterHeader[currentLetter]->getRead();
};

const int	letter_list::getRecieved()
{
	return letterHeader[currentLetter]->getRecieved();
};

const int	letter_list::getPrivate()
{
	return letterHeader[currentLetter]->getPrivate();
};

const int	letter_list::getReplied()
{
	return letterHeader[currentLetter]->getReplied();
};

const int	letter_list::getKilled()
{
	return letterHeader[currentLetter]->getKilled();
};

const int	letter_list::getOriginalID()
{
	return letterHeader[currentLetter]->getOriginalID();
};

const char* letter_list::getBody()
{
	return driver->getBody(currentLetter);
};

const int letter_list::getLength()
{
	return letterHeader[currentLetter]->getLength();
};

int letter_list::setRead()
{
	return letterHeader[currentLetter]->setRead(); 
};
	
int letter_list::setUnread()
{
	return letterHeader[currentLetter]->setUnread();
};

int letter_list::rrefresh()
{
	cleanup();
	init();
	return 0;
};
