/*
 * MultiMAIL offline mail reader
 * 

   Written by Toth Istvan (stoty@vma.bme.hu)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "mmail.h"
#include "resource.h"
#include <iostream.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

// ------------------------------------------------------------------------
// packet_list functions
// ------------------------------------------------------------------------

packet_list::packet_list(mmail *mmA)
{
	mm = mmA;
	initPackets((char*)mm->resourceObject->get(bwPacketDir));		
};

packet_list::~packet_list()
{
	for(int c=1; c<=noOfPackets; c++)
		delete packets[c];
	delete packets;
};

int packet_list::initPackets(char *bwDirName)
{
	DIR  *bwDir;
	struct dirent *entry;
	struct stat fileStat;
	char charTime[255];

	if(!(bwDir=opendir(bwDirName)))
	{
		fatalError("\n\nThere is no Packet Dir!\n (hint: probably you haven't \
run make intstall_dir,\n or the .mmailrc file is corrupt)\n\n");
	};
	
	chdir(bwDirName);
	
	// Lame, but I don't like lists 
	noOfPackets = 0;
	while((entry=readdir(bwDir)))
	 	if(!(stat(entry->d_name, &fileStat)))
			if( (!access(entry->d_name,R_OK)) && (entry->d_name[0]!='.') )
			{
				noOfPackets++;
			};
		
	rewinddir(bwDir);
	packets = new (packet_header*)[noOfPackets+1];

	int c=1;
	while((entry=readdir(bwDir)))
		if(!(stat(entry->d_name, &fileStat)))
			if( (!access(entry->d_name,R_OK)) && (entry->d_name[0]!='.') )
			{
				strcpy(charTime,ctime(&(fileStat.st_mtime)));
				charTime[strlen(charTime)-1]='\0';
				packets[c++] = new packet_header(mm, entry->d_name, 0, 0, 0,
				charTime);
			}; 

	closedir(bwDir);
	return 0;
};

int packet_list::getNoOfPackets()
{
	return noOfPackets;
};

int packet_list::gotoPacket(int packetNo)
{
	if( (packetNo < 1) || (packetNo >noOfPackets) )
		return -1;
	activePacket = packetNo;
	return 0;
};

char *packet_list::getName()
{
	if(noOfPackets)
		return packets[activePacket]->getName();
	else
		return NULL;
}

char* packet_list::getDate()
{
	if(noOfPackets)
		return packets[activePacket]->getDate();
	else 
		return NULL;
};

int packet_list::getNoOfLetters()
{
	if(noOfPackets)
		return packets[activePacket]->getNoOfLetters();
	else 
		return -1;
}

int packet_list::getNoOfUnread()
{
	if(noOfPackets)
		return packets[activePacket]->getNoOfUnread();
	else
		return -1;
};

int packet_list::getNoOfPersonal()
{
	if(noOfPackets)
		return packets[activePacket]->getNoOfPersonal();
	else
		return -1;
};



// --------------------------------------------------------------------------
// packet_header methods
// --------------------------------------------------------------------------
packet_header::packet_header(mmail *mmA,  char* nameA, int noOfLettersA, 
							int noOfUnreadA ,int noOfPersonalA, char* dateA)
{
	mm = mmA;
	name = strdupplus(nameA);
	noOfLetters = noOfLettersA;
	noOfPersonal = noOfPersonalA;
	noOfUnread = noOfUnreadA;
	date = strdupplus(dateA);
}  

packet_header::~packet_header()
{
	delete name;
	delete date;
};

char *packet_header::getName()
{
	return name;
}

char* packet_header::getDate()
{
	return date;
};

int packet_header::getNoOfLetters()
{
	return noOfLetters;
}

int packet_header::getNoOfUnread()
{
	return noOfUnread;
};

int packet_header::getNoOfPersonal()
{
	return noOfPersonal;
};
